@@ -14,7 +14,7 @@
 
 import json
 from copy import copy
-from typing import cast, Iterable, List, Optional
+from typing import cast, Any, Dict, Iterable, List, Optional, Sequence, Union
 from uuid import uuid4
 from logging import warning
 
@@ -34,9 +34,10 @@ from pytket.backends import (
     StatusEnum,
 )
 from pytket.backends.backend import KwargTypes
+from pytket.backends.backendinfo import BackendInfo
 from pytket.backends.backendresult import BackendResult
 from pytket.backends.resulthandle import _ResultIdTuple
-from pytket.device import Device  # type: ignore
+from pytket.extensions.pyquil._metadata import __extension_version__
 from pytket.passes import (  # type: ignore
     BasePass,
     EulerAngleReduction,
@@ -63,6 +64,7 @@ from pytket.predicates import (  # type: ignore
 )
 from pytket.extensions.pyquil.pyquil_convert import (
     process_characterisation,
+    get_avg_characterisation,
     tk_to_pyquil,
 )
 from pytket.routing import NoiseAwarePlacement, Architecture  # type: ignore
@@ -89,6 +91,7 @@ class ForestBackend(Backend):
     _supports_counts = True
     _supports_contextual_optimisation = True
     _persistent_handles = True
+    _GATE_SET = {OpType.CZ, OpType.Rx, OpType.Rz, OpType.Measure, OpType.Barrier}
 
     def __init__(self, qc_name: str, simulator: bool = True):
         """Backend for running circuits on a Rigetti QCS device or simulating with the
@@ -103,11 +106,23 @@ class ForestBackend(Backend):
         """
         super().__init__()
         self._qc: QuantumComputer = get_qc(qc_name, as_qvm=simulator)
-        self._characterisation: dict = process_characterisation(self._qc)
-        self._device: Device = Device(
-            self._characterisation.get("NodeErrors", {}),
-            self._characterisation.get("EdgeErrors", {}),
-            self._characterisation.get("Architecture", Architecture([])),
+
+        char_dict: dict = process_characterisation(self._qc)
+        arch = char_dict.get("Architecture", Architecture([]))
+        node_errors = char_dict.get("NodeErrors")
+        link_errors = char_dict.get("EdgeErrors")
+        self._backend_info = BackendInfo(
+            type(self).__name__,
+            qc_name,
+            __extension_version__,
+            arch,
+            self._GATE_SET,
+            misc={
+                "characterisation": {
+                    "NodeErrors": node_errors,
+                    "EdgeErrors": link_errors,
+                }
+            },
         )
 
     @property
@@ -116,10 +131,8 @@ class ForestBackend(Backend):
             NoClassicalControlPredicate(),
             NoFastFeedforwardPredicate(),
             NoMidMeasurePredicate(),
-            GateSetPredicate(
-                {OpType.CZ, OpType.Rx, OpType.Rz, OpType.Measure, OpType.Barrier}
-            ),
-            ConnectivityPredicate(self._device),
+            GateSetPredicate(self.backend_info.gate_set),
+            ConnectivityPredicate(self.backend_info.architecture),
         ]
 
     def default_compilation_pass(self, optimisation_level: int = 1) -> BasePass:
@@ -134,8 +147,11 @@ class ForestBackend(Backend):
             passlist.append(FullPeepholeOptimise())
         passlist.append(
             CXMappingPass(
-                self._device,
-                NoiseAwarePlacement(self._device),
+                self.backend_info.architecture,
+                NoiseAwarePlacement(
+                    self.backend_info.architecture,
+                    **get_avg_characterisation(self.characterisation),
+                ),
                 directed_cx=False,
                 delay_measures=True,
             )
@@ -162,8 +178,8 @@ class ForestBackend(Backend):
 
     def process_circuits(
         self,
-        circuits: Iterable[Circuit],
-        n_shots: Optional[int] = None,
+        circuits: Sequence[Circuit],
+        n_shots: Optional[Union[int, Sequence[int]]] = None,
         valid_check: bool = True,
         **kwargs: KwargTypes,
     ) -> List[ResultHandle]:
@@ -171,17 +187,30 @@ class ForestBackend(Backend):
         See :py:meth:`pytket.backends.Backend.process_circuits`.
         Supported kwargs: `seed`.
         """
-        if n_shots is None or n_shots < 1:
-            raise ValueError(
-                "Parameter n_shots is required for this backend for ForestBackend"
-            )
+        circuits = list(circuits)
+        n_shots_list: List[int] = []
+        if hasattr(n_shots, "__iter__"):
+            for n in cast(Sequence[Optional[int]], n_shots):
+                if n is None or n < 1:
+                    raise ValueError(
+                        "n_shots values are required for all circuits for this backend"
+                    )
+                n_shots_list.append(n)
+            if len(n_shots_list) != len(circuits):
+                raise ValueError("The length of n_shots and circuits must match")
+        else:
+            if n_shots is None:
+                raise ValueError("Parameter n_shots is required for this backend")
+            # convert n_shots to a list
+            n_shots_list = [cast(int, n_shots)] * len(circuits)
+
         if valid_check:
             self._check_all_circuits(circuits)
 
         postprocess = kwargs.get("postprocess", False)
 
         handle_list = []
-        for circuit in circuits:
+        for circuit, n_shots in zip(circuits, n_shots_list):
             if postprocess:
                 c0, ppcirc = prepare_circuit(circuit, allow_classical=False)
                 ppcirc_rep = ppcirc.to_dict()
@@ -239,18 +268,37 @@ class ForestBackend(Backend):
             return res
 
     @property
-    def characterisation(self) -> Optional[dict]:
-        return self._characterisation
+    def characterisation(self) -> Dict[str, Any]:
+        char = self._backend_info.get_misc("characterisation")
+        return cast(Dict[str, Any], char)
 
     @property
-    def device(self) -> Optional[Device]:
-        return self._device
+    def backend_info(self) -> BackendInfo:
+        return self._backend_info
 
 
 class ForestStateBackend(Backend):
     _supports_state = True
     _supports_expectation = True
+    _expectation_allows_nonhermitian = False
     _persistent_handles = False
+    _GATE_SET = {
+        OpType.X,
+        OpType.Y,
+        OpType.Z,
+        OpType.H,
+        OpType.S,
+        OpType.T,
+        OpType.Rx,
+        OpType.Ry,
+        OpType.Rz,
+        OpType.CZ,
+        OpType.CX,
+        OpType.CCX,
+        OpType.CU1,
+        OpType.U1,
+        OpType.SWAP,
+    }
 
     def __init__(self) -> None:
         """Backend for running simulations on the Rigetti QVM Wavefunction Simulator."""
@@ -264,25 +312,7 @@ class ForestStateBackend(Backend):
             NoFastFeedforwardPredicate(),
             NoMidMeasurePredicate(),
             NoSymbolsPredicate(),
-            GateSetPredicate(
-                {
-                    OpType.X,
-                    OpType.Y,
-                    OpType.Z,
-                    OpType.H,
-                    OpType.S,
-                    OpType.T,
-                    OpType.Rx,
-                    OpType.Ry,
-                    OpType.Rz,
-                    OpType.CZ,
-                    OpType.CX,
-                    OpType.CCX,
-                    OpType.CU1,
-                    OpType.U1,
-                    OpType.SWAP,
-                }
-            ),
+            GateSetPredicate(self._GATE_SET),
             DefaultRegisterPredicate(),
         ]
 
@@ -305,7 +335,7 @@ class ForestStateBackend(Backend):
     def process_circuits(
         self,
         circuits: Iterable[Circuit],
-        n_shots: Optional[int] = None,
+        n_shots: Optional[Union[int, Sequence[int]]] = None,
         valid_check: bool = True,
         **kwargs: KwargTypes,
     ) -> List[ResultHandle]:
@@ -344,10 +374,6 @@ class ForestStateBackend(Backend):
             return CircuitStatus(StatusEnum.COMPLETED)
         raise CircuitNotRunError(handle)
 
-    @property
-    def device(self) -> Optional[Device]:
-        return None
-
     def _gen_PauliTerm(self, term: QubitPauliString, coeff: complex = 1.0) -> PauliTerm:
         pauli_term = ID() * coeff
         for q, p in term.to_dict().items():
